@@ -4,21 +4,35 @@ from urllib.parse import urlparse
 import collections
 #import threading
 
+prefix = "[\033[1;36mlibovsdb\033[0m]: "
 #class OVSDBConnection(threading.Thread):
 class OVSDBConnection(object):
     """ This is a connects to an ovsdb server that has manager set using
         ovs-vsctl set-manager ptcp:5000
     """
-    def __init__(self, server_path, db = None, verbose = False):
-        log_file = "libovsdb_%s.log" %(time.strftime("%Y%m%d%H%M%S", time.localtime()))
-        #print(verbose)
-        logging.basicConfig(
-                #filename = log_file,
-                level= verbose and logging.DEBUG or logging.INFO,
+    def __init__(self, server_path, db = None, **kwargs):
+        self.id = 0
+        self.db_name = db
+        self.server_path = server_path
+        self.attributes = collections.OrderedDict()
+        for k,v in kwargs.items():
+            #print("%s: %s" %(k, v))
+            self[k] = v
+
+        log_file = kwargs.get("log_file", None);
+                #"libovsdb_%s.log" %(time.strftime("%Y%m%d%H%M%S", time.localtime())))
+        logging.basicConfig(filename = log_file,
+                level= kwargs.get("log_level", logging.INFO),
                 format='%(asctime)s%(message)s',
                 datefmt='[%Y%m%d %I:%M:%S]',
                 filemode = 'w')
 
+        logging.debug(prefix + "logging start")
+        if self["dryrun"]:
+            logging.debug(prefix + "dry-fun, not really connect to: %s."
+                    %(server_path))
+            return
+
         #threading.Thread.__init__(self)
         o = urlparse(server_path)
         #print(o)
@@ -31,23 +45,39 @@ class OVSDBConnection(object):
             self.socket = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
             self.socket.connect(o.path)
 
-        self.id = 0
-        self.db_name = db
-        self.server_path = server_path
         if not self.db_name:
             ret = self.list_dbs()
             self.db_name = ret["result"][0]
-        logging.debug("succeed to connected db: %s/%s" %(server_path, self.db_name))
+        logging.debug(prefix + "succeed to connected db: %s/%s" %(server_path, self.db_name))
         #self.start()
 
+    def __getitem__(self, name):
+        """
+        Get members of the object.
+        """
+        return self.attributes.get(name, None)
+
+    def __setitem__(self, name, value):
+        """
+        Set members to the object.
+        """
+        self.attributes[name] = value
+
     def send (self, message, callback=None):
         '''
         send request to the ovsdb server
         '''
         if callback:
             self.callbacks[message['id']] = callback
-        request = json.dumps(message)
-        logging.debug("send message: %s" %(request))
+        request = json.dumps(message, separators=(',',':'))
+
+        if self["dryrun"]:
+            print("ovsdb-client %s %s '%s'"
+                    %(json.dumps(message["method"], separators=(',',':')),
+                        self.server_path,
+                        json.dumps(message["params"], separators=(',',':'))))
+            return
+        logging.debug(prefix + "send message: %s" %(request))
         self.socket.send(request.encode("UTF-8"))
 
     def recv (self, buffer_len = 4096):
@@ -55,8 +85,10 @@ class OVSDBConnection(object):
         recieve response from the ovsdb server. It's in block mode and return
         after receive a complete message or timeout.
         '''
+        if self["dryrun"]:
+            return
         #response = self.socket.recv(buffer_len).decode("UTF-8")
-        #logging.debug("%s" %(response))
+        #logging.debug(prefix + "%s" %(response))
         #return json.loads(response)
         chunks = []
         lc = rc = 0
@@ -64,7 +96,7 @@ class OVSDBConnection(object):
             response = self.socket.recv(4096)
             if response:
                 response = response.decode('utf8')
-                #logging.debug("run() response: %s." %(response))
+                #logging.debug(prefix + "run() response: %s." %(response))
                 message_mark = 0
                 for i, c in enumerate(response):
                     #todo fix the curlies in quotes
@@ -80,7 +112,7 @@ class OVSDBConnection(object):
                         message = "".join(chunks)
                         #print("message: %s" %(message))
                         #self._on_remote_message(message)
-                        logging.debug("recv message: %s" %(message))
+                        logging.debug(prefix + "recv message: %s" %(message))
                         json_m = json.loads(message,
                                 #object_pairs_hook=collections.OrderedDict,
                                 )
@@ -94,7 +126,7 @@ class OVSDBConnection(object):
                 #    print("message not complete, continue receiving, lc: %d, rc: %d..." %(lc, rc))
                 #    print("response: %s" %(response))
                 chunks.append(response[message_mark:])
-                #logging.debug("left message: %s." %(response[message_mark:]))
+                #logging.debug(prefix + "left message: %s." %(response[message_mark:]))
 
     def list_dbs (self, buffer_len = 4096):
         '''
@@ -118,6 +150,83 @@ class OVSDBConnection(object):
         #self.send(list_dbs_query)
         #return db.recv()
 
+    def insert (self, table, row, refer = None, **kwargs):
+        ''' Insert a row into the given table '''
+        tx = self.transact()
+        tx.row_insert(table = table, row = row, refer = refer, **kwargs)
+        res = tx.commit()
+        #print("AAAAAAAAAAAAAAAAAAAAA, res: %s" %(res))
+        if not res.get("result", None):
+            return None
+
+        #item = { "uuid": [], "details": None, "error": None, } #init dict
+        item = { "uuid": []}
+        for r in res["result"]:
+            if isinstance(r, dict):
+                for k,v in r.items():
+                    value = isinstance(v, list) and v[1] or v
+                    if k == "uuid":
+                        item[k].append(value)
+                    else:
+                        item[k] = value
+        return item
+
+    def delete (self, table, where, **kwargs):
+        ''' Delete a row from the given table '''
+        tx = self.transact()
+        tx.row_delete(table = table, where = where)
+        res = tx.commit()
+        if not res.get("result", None):
+            return None
+
+        item = collections.OrderedDict()
+        for r in res["result"]:
+            for k,v in r.items():
+                item[k] = isinstance(v, list) and v[1] or v
+        return item
+
+    def update (self, table, row, where, **kwargs):
+        ''' update a row in the given table '''
+        tx = self.transact()
+        tx.row_update(table = table, row = row, where = where)
+
+        res = tx.commit()
+        if not res.get("result", None):
+            return None
+
+        item = collections.OrderedDict()
+        for r in res["result"]:
+            for k,v in r.items():
+                item[k] = isinstance(v, list) and v[1] or v
+        return item
+
+    def select (self, table, where, **kwargs):
+        ''' select a row in the given table '''
+        tx = self.transact()
+        tx.row_select(table = table, where = where)
+        res = tx.commit()
+        if not res.get("result", None):
+            return None
+
+        result = []
+        for row in res["result"][0]["rows"]:
+            item = collections.OrderedDict()
+            for k,v in row.items():
+                if isinstance(v, list):
+                    # Remove leading "uuid"
+                    item[k] = []
+                    for l in v[1]:
+                        if isinstance(l, list):
+                            # Remove leading "uuid"
+                            item[k].append(l[1])
+                        else:
+                            item[k] = l
+                else:
+                    item[k] = v
+            result.append(item)
+        #return res["result"][0]["rows"]
+        return result
+
 class OVSDBTransact(object):
     """transact
     """
@@ -131,7 +240,7 @@ class OVSDBTransact(object):
         '''
         Recycle resource when the object is destroied.
         '''
-        print("transact %d, finish in %.2f seconds\n"
+        logging.debug(prefix + "transact %d, finish in %.2f seconds\n"
                 %(self.ovsdb_connection.id, time.time() - self.start_time))
 
     def insert (self, table, row, op = "insert", **kwargs):
@@ -292,19 +401,20 @@ class OVSDBTransact(object):
         '''
         mutatesets = []
         # It seems that json don't support dump python set, so conver it to
-        # list
+        # list, in other words, conver {"key": "value"} to ["key": "value"]
         for a in args:
-            mutateset = []
+            #print("aaaaaaaaa: %s, type: %s" %(a, type(a)))
             if isinstance(a, dict):
+                mutateset = []
                 #{"named-uuid": name}
                 for k,v in a.items():
                     mutateset.append(k)
                     mutateset.append(v)
-            elif isinstance(a, list):
+            else:
                 #['uuid', '433fb2e1-fd87-4a40-b3ef-991dcd7b8014']
                 mutateset = a
-            #print("mutatesets: %s, a: %s, list(a): %s" %(mutatesets, a, list(a)))
             mutatesets.append(mutateset)
+            #print("mutatesets: %s" %(mutatesets))
         mutations = [item, mutator, ["set", mutatesets]]
         return mutations
 
@@ -326,49 +436,60 @@ class OVSDBTransact(object):
         #self.start_time = time.time() # reset the timer
         return response
 
-    def row_insert (self, table, row, referby = None):
+    def row_insert (self, table, row, refer = None):
         '''
         Insert a row, if there is refTable by others, mutate it. It's a
         wrapper for self.insert() to let caller insert a row in one line. an
         example:
             tx.row_insert(table = "Logical_Switch_Port", row = {"name":"ls1-lsp0"},
-                    referby = ["Logical_Switch", "ls1", "ports"])
+                    refer = ["Logical_Switch", "ports", ["name", "==", "ls1"]])
         '''
-        name = self.insert(table = table, row = row)
-        if referby: #It's a refTable
-            self.mutate(table = referby[0],
-                    where = [["name", "==", referby[1]]],
-                    mutations = [self.make_mutations(referby[2], "insert", {"named-uuid": name})])
+        uuid_names = []
+        rows = isinstance(row, list) and row or [row]
+        for r in rows:
+            name = self.insert(table = table, row = r)
+            if refer: #It's a refTable
+                uuid_names.append({"named-uuid": name})
+        #print(uuid_names)
+        if refer: #It's a refTable
+            mutations = self.make_mutations(refer[1], "insert", *uuid_names)
+            self.mutate(table = refer[0],
+                    where = [refer[2]],
+                    mutations = [mutations])
 
-    def row_delete (self, table, where = None, referby = None):
+    def row_delete (self, table, where = None, refer = None):
         '''
         Delete a row, if there is refTable by others, mutate it. It's a
         wrapper for self.delete() to let caller delete a row in one line. for
         example:
             tx.row_delete(table = "Logical_Switch_Port",
                     where = [["name", "==", "ls1-lsp0"]],
-                    referby = ["Logical_Switch", "ls1", "ports"])
+                    refer = ["Logical_Switch", "ls1", "ports"])
         '''
-        # Get the uuid at frist it's needed in mutations.
-        select_op = {"op": "select",
-                "table": table,
-                "columns": ["_uuid", "name"]}
-        select_op["where"] = where and where or []
-        list_dbs_query = {"method":"transact",
-                "params": [self.ovsdb_connection.db_name, select_op],
-                "id": self.ovsdb_connection.id}
-        self.ovsdb_connection.send(list_dbs_query)
-        self.ovsdb_connection.id += 1
-        response = self.ovsdb_connection.recv()
-        uuid = []
-        for row in response["result"][0]["rows"]:
-            uuid.append(row["_uuid"])
-            #print(uuid)
 
-        #self.delete(table = table, where = where) #delete op is not needed???
-        if referby: #It's a refTable
-            self.mutate(table = referby[0], where = [["name", "==", referby[1]]],
-                    mutations = [self.make_mutations(referby[2], "delete", *uuid)])
+        if not refer: #It's a refTable
+            #print("not refer")
+            self.delete(table = table, where = where) #delete op is not needed???
+        else: #It's a refTable
+            # select the uuid at frist since it's needed in mutations.
+            select_op = {"op": "select",
+                    "table": table,
+                    "columns": ["_uuid"]}
+            select_op["where"] = where and where or []
+            list_dbs_query = {"method":"transact",
+                    "params": [self.ovsdb_connection.db_name, select_op],
+                    "id": self.ovsdb_connection.id}
+            self.ovsdb_connection.send(list_dbs_query)
+            self.ovsdb_connection.id += 1
+            response = self.ovsdb_connection.recv()
+            uuid = []
+            for row in response["result"][0]["rows"]:
+                uuid.append(row["_uuid"])
+                #print(uuid)
+            self.mutate(table = refer[0], where = [refer[2]],
+                    mutations = [self.make_mutations(refer[1],
+                                                    "delete",
+                                                    *uuid)])
 
     def row_update (self, table, row, where = None):
         '''
@@ -396,7 +517,7 @@ if __name__ == '__main__':
         #tx.row_insert(table = "Logical_Switch", row = {"name":"ls1"})
         tx.row_insert(table = "Logical_Switch_Port",
                 row = {"name":"ls1-lsp0"},
-                referby = ["Logical_Switch", "ls1", "ports"])
+                refer = ["Logical_Switch", "ports", ["name", "==", "ls1"]])
 
     if options.update:
         tx.row_update(table = "Logical_Switch_Port",
